package cn.com.qmth.am.task;

import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.concurrent.service.ConcurrentService;

import cn.com.qmth.am.config.SysProperty;
import cn.com.qmth.am.entity.StudentEntity;
import cn.com.qmth.am.entity.StudentScoreEntity;
import cn.com.qmth.am.enums.LockType;
import cn.com.qmth.am.service.StudentScoreService;
import cn.com.qmth.am.service.StudentService;

@Service
public class AiMarkingJob {
	@Autowired
	private StudentScoreService studentScoreService;
	@Autowired
	private StudentService studentService;
	@Autowired
	private ConcurrentService concurrentService;
	@Autowired
	private SysProperty sysProperty;

	@Scheduled(fixedDelay = 5 * 1000, initialDelay = 20 * 1000)
	public void doJob() {
		if(!sysProperty.getMarkingTaskEnable()) {
			return;
		}
		List<StudentEntity> students=studentService.findToMrking();
		if (CollectionUtils.isEmpty(students)) {
			return;
		}
		boolean lock = concurrentService.getReadWriteLock(LockType.AI_MARKING.name()).writeLock().tryLock();
		try {
			if (!lock) {
				return;
			}
			this.dispose(students);
		} finally {
			if (lock) {
				concurrentService.getReadWriteLock(LockType.AI_MARKING.name()).writeLock().unlock();
			}
		}
	}

	private void dispose(List<StudentEntity> students) {
		for (StudentEntity student : students) {
			if(!sysProperty.getMarkingTaskEnable()) {
				return;
			}
			List<StudentScoreEntity> scores = studentScoreService.findToAiMarking(student.getId());
			if (CollectionUtils.isNotEmpty(scores)) {
				for (StudentScoreEntity score : scores) {
					studentScoreService.aiMarking(score);
				}
			}
		}
	}

}
